# Template C++ Project

[![Language Server: Clangd](https://img.shields.io/badge/Language%20Server-Clangd-blue.svg)](https://clangd.llvm.org/)
[![Code Formatter: ClangFormat](https://img.shields.io/badge/Code%20Formatter-ClangFormat-blue.svg)](https://clang.llvm.org/docs/ClangFormat.html)

## VSCode Project Settings

In order to ensure that everyone uses the same linting and formatting settings,
we duplicate the relevant settings in [`.vscode/settings.json`](.vscode/settings.json).

Relevant settings with c++ :

- format on save
- using `clangd` language server over the standard one
- using `clang-format` c++ formatter

## Formatter - ClangFormat

[ClangFormat](https://clang.llvm.org/docs/ClangFormat.html) is a good c++ formatter.
Using a formatter ensures that the code you and your team write is consistent.
You should always use a formatter.

You can edit the formatter style using the `clang-format.style` setting in [`settings.json`](.vscode/settings.json).

## Language Server - Clangd

The [Clangd](https://clangd.llvm.org/) language server is a better language server than the default one used by VSCode C++ extensions. It also provides inlay hints. In order for it to work, you must install the Clangd extension, and disable the default C++ extension's intellisense.

## Environment and reproducibility

TODO: linux

In order to make it easier for other developers to contribute to your project,
and to enable deploying your application easily, you must indicate which python version and dependencies you are using.

You typically indicate the python version in your [`pyproject.toml`](./pyproject.toml) or (`README.md`)(./README.md) file.

### Git

Some common gitignores for a C++ project :

- `build/` : the build output folder.
- `.cache/` : clangd cache folder.

## Python Clean Code

TODO : code rules

Some often-overlooked python clean code rules :

### Docstrings

Always write docstrings in order to describe the purpose of your functions and classes.

```python
def function():
    """This function does something."""
    pass
```

### Type hinting

When you can (always), use type hints to indicate the types of the parameters of your functions, their return types, etc...
You sometimes need to use the `typing` module to indicate more complex types.

```python
from typing import List
def add_to_list(my_list: List[int], element: int) -> List[int]:
    my_list.append(element)
    return my_list
```

Since python 3.10, you can use the `|` operator to indicate that a parameter can be of multiple types, and use `list` and `dict` directly as type hints.

```python
def add_to_list(my_list: list[int | float], element: int | float) -> list[int | float]:
    my_list.append(element)
    return my_list
```

### Use modules

When your code gets big, you should split it into multiple files.

Use the following test in order to run module-specific code only when you run your module directly via `python module.py`, and not when you import it from another file.

```python
def module_function():
    pass

if __name__ == "__main__":
    module_function()
```
